---
title: "Processing and Exporting Data"
weight: 4
---

Once you've instrumented your code, you need to get the data out in order to do anything useful with it. This page will cover the basics of the process and export pipeline.

# Sampling

Sampling is a process that restricts the amount of traces that are generated by a system. The exact sampler you should use depends on your specific needs, but in general you should make a decision at the start of a trace, and allow the sampling decision to propagate to other services.

A sampler needs to be set on the tracer provider when its configured, as follows:

```go
provider := sdktrace.NewTracerProvider(
  sdktrace.WithConfig(sdktrace.Config{DefaultSampler: sdktrace.AlwaysSample()}),
)
```

`AlwaysSample` and `NeverSample` are fairly self-explanatory. Always means that every trace will be sampled, the converse holds as true for Never. When you're getting started, or in a development environment, you'll almost always want to use `AlwaysSample`.

Other samplers include:

* `TraceIDRatioBased`, which will sample a fraction of traces, based on the fraction given to the sampler. Thus, if you set this to .5, half of traces will be sampled.
* `ParentBased`, which behaves differently based on the incoming sampling decision. In general, this will sample spans that have parents that were sampled, and will not sample spans whose parents were _not_ sampled.

# Resources

Resources are a special type of attribute that apply to all spans generated by a process. These should be used to represent underlying metadata about a process that's non-ephemeral - for example, the hostname of a process, or its instance ID.

Resources should be assigned to a tracer provider at its initialization, and are created much like attributes:

```go
resources := resource.New(
  label.String("service.name", "myService"),
  label.String("service.version", "1.0.0"),
  label.String("instance.id", "abcdef12345"),
)

provider := sdktrace.NewTracerProvider(
  ...
  sdktrace.WithResources(resources),
)
```

# OTLP Exporter

OpenTelemetry Protocol (OTLP) is available in the `go.opentelemetry.io/otel/exporters/otlp` package.

Please find more documentation on [GitHub](https://github.com/open-telemetry/opentelemetry-go/tree/main/exporters/otlp)

# Jaeger Exporter

Jaeger export is available in the `go.opentelemetry.io/otel/exporters/trace/jaeger` package.

Please find more documentation on [GitHub](https://github.com/open-telemetry/opentelemetry-go/tree/main/exporters/trace/jaeger)

# Prometheus Exporter

Prometheus export is available in the `go.opentelemetry.io/otel/exporters/metric/prometheus` package.

Please find more documentation on [GitHub](https://github.com/open-telemetry/opentelemetry-go/tree/main/exporters/metric/prometheus)
